#ifndef __GENIEDEFS__
#define __GENIEDEFS__

#if defined(__unix__) && !defined(__unix)
#define __unix 1
#endif 

#if HAVE_LIBDCE
#    define DAE_SUPPORT		1
#endif

#if defined(__VMS) && HAVE_LIBCMA
#    define HAVE_LIBPTHREADS 	1
#    define DAE_SUPPORT		1
#endif

/* Include gnu autoconfig header file to sort out versioning */
#ifdef HAVE_CONFIG_H
#	include "../config.h"
#endif

/* This include file must really be at the top of everything if it is used at all */
#ifdef HAVE_LIBPTHREADS
#	include <pthread.h>
#endif

/* General definitions which pervade all GENIE source, not specific to the
   parser or the smalltalk interpreter */

#define LONGCHARBUFF	13200	    /* Should hold at least 100 lines of 132 column text */
#define SHORTCHARBUFF	132	    /* One screen widths worth of 132 column text */


/* This breaks the rule of not nesting includes but is done to ensure that anything
   including geniedefs.h also gets the genie IO interface.  All the includes here are
   essential for iofuncs.h and hence will be included for the whole of Open GENIE.
*/

#include <errno.h>
#include <stdio.h>
#ifdef __cplusplus
#include "gnustring.h"
#   include <iostream.h>
#	ifdef _WIN32
#		include <strstrea.h>
#	else
#		include <strstream.h>
#	endif
#   include <iomanip.h>
#   include "genie_ostream.h"
#endif

/* Include this before iofuncs.h because stdlib defines perror */
#ifdef _WIN32
#	include <stdlib.h>
#endif

/* Avoid this for the data access header file */
#ifndef __GENIE_DATA_ACCESS__
#   include "iofuncs.h"
#endif

extern char version[];  /* GENIE wide versioning available */

#define MAIL_GENIE_BUG 	"Please email a bug report to genie@isise.rl.ac.uk"

/* 
 *  This is taken from the CVS manual. If you have used a line of the form:
 *
 *   		static const char* rcsid = "$Id: geniedefs.h,v 1.25 1997/06/25 08:33:07 cmm Exp $";
 *
 * Then you should add the line:
 * 
 *		USE_RCSID(rcsid);
 *
 * Immediately below it so (1) it is not optimised away and (2) so you get no "unused variable" warnings
 */ 

/* #define USE_RCSID(rcsid)	static void* use_##rcsid = (&use_##rcsid, (void*)&rcsid) */
#define USE_RCSID(rcsid)	static void* use_##rcsid = (void*)&rcsid

#if (defined(vms) || defined(VMS)) && !defined(__VMS)
#define __VMS
#endif

/* 
 * some file system specific definitions such as the
 * character we need to insert when concatenating a directory
 * and a file together to make a complete path 
 */
#if defined(__VMS)
#define DIRECTORY_SEPARATOR	""
#define NULL_OUTPUT_DEVICE	"NL:"
#define CURRENT_DIRECTORY	"SYS$DISK:[]"
#elif defined(__WIN32)
#define DIRECTORY_SEPARATOR	"\\"
#define NULL_OUTPUT_DEVICE	"/dev/null"
#define CURRENT_DIRECTORY	"."
#else	/* unix */
#define DIRECTORY_SEPARATOR	"/"
#define NULL_OUTPUT_DEVICE	"/dev/null"
#define CURRENT_DIRECTORY	"."
#endif
#define CURRENT_DIRECTORY_PATH	CURRENT_DIRECTORY DIRECTORY_SEPARATOR

/* Declare the parser process (extern "C" only for linking on VMS) */
#ifdef __cplusplus 
extern "C"
int parser_main(int argc, char *argv[], char *arge[],
                istream &in,
                genie_ostream &out, genie_ostream &inf,
                genie_ostream &err, genie_ostream &dbg);

#endif

/* 
 * Replace sigsetjmp() and siglongjmp() if we lack them
 */
#ifndef HAVE_SIGJMP_BUF
#define sigjmp_buf jmp_buf
#define siglongjmp longjmp
#define sigsetjmp(__env,__savemask)	setjmp(__env)
#endif /* HAVE_SIGJMP_BUF */

#endif /* __GENIEDEFS__ */
